@@ -42,7 +42,11 @@
 //!
 //! println!("Next PP: {}", next_result.pp());
 //!
-//! let stars = map.stars(16, None).stars(); // HR
+//! let stars = map.stars()
+//!     .mods(16)  // HR
+//!     .calculate()
+//!     .stars();
+//!
 //! let max_pp = map.max_pp(16).pp();
 //!
 //! println!("Stars: {} | Max PP: {}", stars, max_pp);
@@ -159,11 +163,7 @@
 //!
 //! | Flag | Description |
 //! |-----|-----|
-//! | `default` | Enable all modes. |
-//! | `osu` | Enable osu!standard. |
-//! | `taiko` | Enable osu!taiko. |
-//! | `fruits` | Enable osu!ctb. |
-//! | `mania` | Enable osu!mania. |
+//! | `default` | Beatmap parsing will be non-async |
 //! | `async_tokio` | Beatmap parsing will be async through [tokio](https://github.com/tokio-rs/tokio) |
 //! | `async_std` | Beatmap parsing will be async through [async-std](https://github.com/async-rs/async-std) |
 //!
@@ -179,23 +179,15 @@
     missing_docs
 )]
 
-#[cfg(feature = "fruits")]
-#[cfg_attr(docsrs, doc(cfg(feature = "fruits")))]
-/// Everything about osu!ctb.
-pub mod fruits;
+/// Everything about osu!catch.
+pub mod catch;
 
-#[cfg(feature = "mania")]
-#[cfg_attr(docsrs, doc(cfg(feature = "mania")))]
 /// Everything about osu!mania.
 pub mod mania;
 
-#[cfg(feature = "osu")]
-#[cfg_attr(docsrs, doc(cfg(feature = "osu")))]
 /// Everything about osu!standard.
 pub mod osu;
 
-#[cfg(feature = "taiko")]
-#[cfg_attr(docsrs, doc(cfg(feature = "taiko")))]
 /// Everything about osu!taiko.
 pub mod taiko;
 
@@ -208,26 +200,20 @@ pub use gradual::{GradualDifficultyAttributes, GradualPerformanceAttributes, Sco
 mod pp;
 pub use pp::{AnyPP, AttributeProvider};
 
+mod stars;
+pub use stars::AnyStars;
+
 mod curve;
 mod mods;
 
-#[cfg(feature = "sliders")]
 pub(crate) mod control_point_iter;
 
-#[cfg(feature = "sliders")]
 pub(crate) use control_point_iter::{ControlPoint, ControlPointIter};
 
-#[cfg(feature = "fruits")]
-pub use fruits::FruitsPP;
-
-#[cfg(feature = "mania")]
-pub use mania::ManiaPP;
-
-#[cfg(feature = "osu")]
-pub use osu::OsuPP;
-
-#[cfg(feature = "taiko")]
-pub use taiko::TaikoPP;
+pub use catch::{CatchPP, CatchStars};
+pub use mania::{ManiaPP, ManiaStars};
+pub use osu::{OsuPP, OsuStars};
+pub use taiko::{TaikoPP, TaikoStars};
 
 pub use mods::Mods;
 pub use parse::{Beatmap, BeatmapAttributes, GameMode, ParseError, ParseResult};
@@ -235,7 +221,7 @@ pub use parse::{Beatmap, BeatmapAttributes, GameMode, ParseError, ParseResult};
 /// Provides some additional methods on [`Beatmap`](crate::Beatmap).
 pub trait BeatmapExt {
     /// Calculate the stars and other attributes of a beatmap which are required for pp calculation.
-    fn stars(&self, mods: impl Mods, passed_objects: Option<usize>) -> DifficultyAttributes;
+    fn stars(&self) -> AnyStars<'_>;
 
     /// Calculate the max pp of a beatmap.
     ///
@@ -252,7 +238,7 @@ pub trait BeatmapExt {
     /// instead of evaluating the final strains, they are just returned as is.
     ///
     /// Suitable to plot the difficulty of a map over time.
-    fn strains(&self, mods: impl Mods) -> Strains;
+    fn strains(&self, mods: u32) -> Strains;
 
     /// Return an iterator that gives you the `DifficultyAttributes` after each hit object.
     ///
@@ -268,69 +254,27 @@ pub trait BeatmapExt {
 
 impl BeatmapExt for Beatmap {
     #[inline]
-    fn stars(&self, mods: impl Mods, passed_objects: Option<usize>) -> DifficultyAttributes {
+    fn stars(&self) -> AnyStars<'_> {
         match self.mode {
-            GameMode::STD => {
-                #[cfg(not(feature = "osu"))]
-                panic!("`osu` feature is not enabled");
-
-                #[cfg(feature = "osu")]
-                DifficultyAttributes::Osu(osu::stars(self, mods, passed_objects))
-            }
-            GameMode::MNA => {
-                #[cfg(not(feature = "mania"))]
-                panic!("`mania` feature is not enabled");
-
-                #[cfg(feature = "mania")]
-                DifficultyAttributes::Mania(mania::stars(self, mods, passed_objects))
-            }
-            GameMode::TKO => {
-                #[cfg(not(feature = "taiko"))]
-                panic!("`taiko` feature is not enabled");
-
-                #[cfg(feature = "taiko")]
-                DifficultyAttributes::Taiko(taiko::stars(self, mods, passed_objects))
-            }
-            GameMode::CTB => {
-                #[cfg(not(feature = "fruits"))]
-                panic!("`fruits` feature is not enabled");
-
-                #[cfg(feature = "fruits")]
-                DifficultyAttributes::Fruits(fruits::stars(self, mods, passed_objects))
-            }
+            GameMode::STD => AnyStars::Osu(OsuStars::new(self)),
+            GameMode::MNA => AnyStars::Mania(ManiaStars::new(self)),
+            GameMode::TKO => AnyStars::Taiko(TaikoStars::new(self)),
+            GameMode::CTB => AnyStars::Catch(CatchStars::new(self)),
         }
     }
 
     #[inline]
     fn max_pp(&self, mods: u32) -> PerformanceAttributes {
         match self.mode {
-            GameMode::STD => {
-                #[cfg(not(feature = "osu"))]
-                panic!("`osu` feature is not enabled");
-
-                #[cfg(feature = "osu")]
-                PerformanceAttributes::Osu(OsuPP::new(self).mods(mods).calculate())
-            }
+            GameMode::STD => PerformanceAttributes::Osu(OsuPP::new(self).mods(mods).calculate()),
             GameMode::MNA => {
-                #[cfg(not(feature = "mania"))]
-                panic!("`mania` feature is not enabled");
-
-                #[cfg(feature = "mania")]
                 PerformanceAttributes::Mania(ManiaPP::new(self).mods(mods).calculate())
             }
             GameMode::TKO => {
-                #[cfg(not(feature = "taiko"))]
-                panic!("`taiko` feature is not enabled");
-
-                #[cfg(feature = "taiko")]
                 PerformanceAttributes::Taiko(TaikoPP::new(self).mods(mods).calculate())
             }
             GameMode::CTB => {
-                #[cfg(not(feature = "fruits"))]
-                panic!("`fruits` feature is not enabled");
-
-                #[cfg(feature = "fruits")]
-                PerformanceAttributes::Fruits(FruitsPP::new(self).mods(mods).calculate())
+                PerformanceAttributes::Catch(CatchPP::new(self).mods(mods).calculate())
             }
         }
     }
@@ -341,36 +285,12 @@ impl BeatmapExt for Beatmap {
     }
 
     #[inline]
-    fn strains(&self, mods: impl Mods) -> Strains {
+    fn strains(&self, mods: u32) -> Strains {
         match self.mode {
-            GameMode::STD => {
-                #[cfg(not(feature = "osu"))]
-                panic!("`osu` feature is not enabled");
-
-                #[cfg(feature = "osu")]
-                osu::strains(self, mods)
-            }
-            GameMode::MNA => {
-                #[cfg(not(feature = "mania"))]
-                panic!("`mania` feature is not enabled");
-
-                #[cfg(feature = "mania")]
-                mania::strains(self, mods)
-            }
-            GameMode::TKO => {
-                #[cfg(not(feature = "taiko"))]
-                panic!("`taiko` feature is not enabled");
-
-                #[cfg(feature = "taiko")]
-                taiko::strains(self, mods)
-            }
-            GameMode::CTB => {
-                #[cfg(not(feature = "fruits"))]
-                panic!("`fruits` feature is not enabled");
-
-                #[cfg(feature = "fruits")]
-                fruits::strains(self, mods)
-            }
+            GameMode::STD => OsuStars::new(self).mods(mods).strains(),
+            GameMode::MNA => ManiaStars::new(self).mods(mods).strains(),
+            GameMode::TKO => TaikoStars::new(self).mods(mods).strains(),
+            GameMode::CTB => CatchStars::new(self).mods(mods).strains(),
         }
     }
 
@@ -398,16 +318,12 @@ pub struct Strains {
 /// The result of a difficulty calculation based on the mode.
 #[derive(Clone, Debug)]
 pub enum DifficultyAttributes {
-    #[cfg(feature = "fruits")]
-    /// osu!ctb difficulty calculation reseult.
-    Fruits(fruits::FruitsDifficultyAttributes),
-    #[cfg(feature = "mania")]
+    /// osu!catch difficulty calculation reseult.
+    Catch(catch::CatchDifficultyAttributes),
     /// osu!mania difficulty calculation reseult.
     Mania(mania::ManiaDifficultyAttributes),
-    #[cfg(feature = "osu")]
     /// osu!standard difficulty calculation reseult.
     Osu(osu::OsuDifficultyAttributes),
-    #[cfg(feature = "taiko")]
     /// osu!taiko difficulty calculation reseult.
     Taiko(taiko::TaikoDifficultyAttributes),
 }
@@ -417,13 +333,9 @@ impl DifficultyAttributes {
     #[inline]
     pub fn stars(&self) -> f64 {
         match self {
-            #[cfg(feature = "fruits")]
-            Self::Fruits(attributes) => attributes.stars,
-            #[cfg(feature = "mania")]
+            Self::Catch(attributes) => attributes.stars,
             Self::Mania(attributes) => attributes.stars,
-            #[cfg(feature = "osu")]
             Self::Osu(attributes) => attributes.stars,
-            #[cfg(feature = "taiko")]
             Self::Taiko(attributes) => attributes.stars,
         }
     }
@@ -432,43 +344,23 @@ impl DifficultyAttributes {
     ///
     /// This will only be `None` for attributes of osu!mania maps.
     #[inline]
-    #[cfg(feature = "mania")]
     pub fn max_combo(&self) -> Option<usize> {
         match self {
-            #[cfg(feature = "fruits")]
-            Self::Fruits(attributes) => Some(attributes.max_combo()),
+            Self::Catch(attributes) => Some(attributes.max_combo()),
             Self::Mania(_) => None,
-            #[cfg(feature = "osu")]
             Self::Osu(attributes) => Some(attributes.max_combo),
-            #[cfg(feature = "taiko")]
             Self::Taiko(attributes) => Some(attributes.max_combo),
         }
     }
+}
 
-    #[cfg(not(feature = "mania"))]
+impl From<catch::CatchDifficultyAttributes> for DifficultyAttributes {
     #[inline]
-    /// The maximum combo of the map.
-    pub fn max_combo(&self) -> usize {
-        match self {
-            #[cfg(feature = "fruits")]
-            Self::Fruits(attributes) => attributes.max_combo,
-            #[cfg(feature = "osu")]
-            Self::Osu(attributes) => attributes.max_combo,
-            #[cfg(feature = "taiko")]
-            Self::Taiko(attributes) => attributes.max_combo,
-        }
+    fn from(attributes: catch::CatchDifficultyAttributes) -> Self {
+        Self::Catch(attributes)
     }
 }
 
-#[cfg(feature = "fruits")]
-impl From<fruits::FruitsDifficultyAttributes> for DifficultyAttributes {
-    #[inline]
-    fn from(attributes: fruits::FruitsDifficultyAttributes) -> Self {
-        Self::Fruits(attributes)
-    }
-}
-
-#[cfg(feature = "mania")]
 impl From<mania::ManiaDifficultyAttributes> for DifficultyAttributes {
     #[inline]
     fn from(attributes: mania::ManiaDifficultyAttributes) -> Self {
@@ -476,7 +368,6 @@ impl From<mania::ManiaDifficultyAttributes> for DifficultyAttributes {
     }
 }
 
-#[cfg(feature = "osu")]
 impl From<osu::OsuDifficultyAttributes> for DifficultyAttributes {
     #[inline]
     fn from(attributes: osu::OsuDifficultyAttributes) -> Self {
@@ -484,7 +375,6 @@ impl From<osu::OsuDifficultyAttributes> for DifficultyAttributes {
     }
 }
 
-#[cfg(feature = "taiko")]
 impl From<taiko::TaikoDifficultyAttributes> for DifficultyAttributes {
     #[inline]
     fn from(attributes: taiko::TaikoDifficultyAttributes) -> Self {
@@ -495,16 +385,12 @@ impl From<taiko::TaikoDifficultyAttributes> for DifficultyAttributes {
 /// The result of a performance calculation based on the mode.
 #[derive(Clone, Debug)]
 pub enum PerformanceAttributes {
-    #[cfg(feature = "fruits")]
-    /// osu!ctb performance calculation result.
-    Fruits(fruits::FruitsPerformanceAttributes),
-    #[cfg(feature = "mania")]
+    /// osu!catch performance calculation result.
+    Catch(catch::CatchPerformanceAttributes),
     /// osu!mania performance calculation result.
     Mania(mania::ManiaPerformanceAttributes),
-    #[cfg(feature = "osu")]
     /// osu!standard performance calculation result.
     Osu(osu::OsuPerformanceAttributes),
-    #[cfg(feature = "taiko")]
     /// osu!taiko performance calculation result.
     Taiko(taiko::TaikoPerformanceAttributes),
 }
@@ -514,13 +400,9 @@ impl PerformanceAttributes {
     #[inline]
     pub fn pp(&self) -> f64 {
         match self {
-            #[cfg(feature = "fruits")]
-            Self::Fruits(attributes) => attributes.pp,
-            #[cfg(feature = "mania")]
+            Self::Catch(attributes) => attributes.pp,
             Self::Mania(attributes) => attributes.pp,
-            #[cfg(feature = "osu")]
             Self::Osu(attributes) => attributes.pp,
-            #[cfg(feature = "taiko")]
             Self::Taiko(attributes) => attributes.pp,
         }
     }
@@ -529,13 +411,9 @@ impl PerformanceAttributes {
     #[inline]
     pub fn stars(&self) -> f64 {
         match self {
-            #[cfg(feature = "fruits")]
-            Self::Fruits(attributes) => attributes.stars(),
-            #[cfg(feature = "mania")]
+            Self::Catch(attributes) => attributes.stars(),
             Self::Mania(attributes) => attributes.stars(),
-            #[cfg(feature = "osu")]
             Self::Osu(attributes) => attributes.stars(),
-            #[cfg(feature = "taiko")]
             Self::Taiko(attributes) => attributes.stars(),
         }
     }
@@ -544,73 +422,45 @@ impl PerformanceAttributes {
     #[inline]
     pub fn difficulty_attributes(&self) -> DifficultyAttributes {
         match self {
-            #[cfg(feature = "fruits")]
-            Self::Fruits(attributes) => DifficultyAttributes::Fruits(attributes.difficulty.clone()),
-            #[cfg(feature = "mania")]
+            Self::Catch(attributes) => DifficultyAttributes::Catch(attributes.difficulty.clone()),
             Self::Mania(attributes) => DifficultyAttributes::Mania(attributes.difficulty),
-            #[cfg(feature = "osu")]
             Self::Osu(attributes) => DifficultyAttributes::Osu(attributes.difficulty.clone()),
-            #[cfg(feature = "taiko")]
             Self::Taiko(attributes) => DifficultyAttributes::Taiko(attributes.difficulty),
         }
     }
 
-    #[cfg(feature = "mania")]
     #[inline]
     /// The maximum combo of the map.
     ///
     /// This will only be `None` for attributes of osu!mania maps.
     pub fn max_combo(&self) -> Option<usize> {
         match self {
-            #[cfg(feature = "fruits")]
-            Self::Fruits(f) => Some(f.difficulty.max_combo()),
+            Self::Catch(f) => Some(f.difficulty.max_combo()),
             Self::Mania(_) => None,
-            #[cfg(feature = "osu")]
             Self::Osu(o) => Some(o.difficulty.max_combo),
-            #[cfg(feature = "taiko")]
             Self::Taiko(t) => Some(t.difficulty.max_combo),
         }
     }
-
-    #[cfg(not(feature = "mania"))]
-    #[inline]
-    /// The maximum combo of the map.
-    pub fn max_combo(&self) -> usize {
-        match self {
-            #[cfg(feature = "fruits")]
-            Self::Fruits(f) => f.difficulty.max_combo,
-            #[cfg(feature = "osu")]
-            Self::Osu(o) => o.difficulty.max_combo,
-            #[cfg(feature = "taiko")]
-            Self::Taiko(t) => t.difficulty.max_combo,
-        }
-    }
 }
 
 impl From<PerformanceAttributes> for DifficultyAttributes {
     fn from(attributes: PerformanceAttributes) -> Self {
         match attributes {
-            #[cfg(feature = "fruits")]
-            PerformanceAttributes::Fruits(attributes) => Self::Fruits(attributes.difficulty),
-            #[cfg(feature = "mania")]
+            PerformanceAttributes::Catch(attributes) => Self::Catch(attributes.difficulty),
             PerformanceAttributes::Mania(attributes) => Self::Mania(attributes.difficulty),
-            #[cfg(feature = "osu")]
             PerformanceAttributes::Osu(attributes) => Self::Osu(attributes.difficulty),
-            #[cfg(feature = "taiko")]
             PerformanceAttributes::Taiko(attributes) => Self::Taiko(attributes.difficulty),
         }
     }
 }
 
-#[cfg(feature = "fruits")]
-impl From<fruits::FruitsPerformanceAttributes> for PerformanceAttributes {
+impl From<catch::CatchPerformanceAttributes> for PerformanceAttributes {
     #[inline]
-    fn from(attributes: fruits::FruitsPerformanceAttributes) -> Self {
-        Self::Fruits(attributes)
+    fn from(attributes: catch::CatchPerformanceAttributes) -> Self {
+        Self::Catch(attributes)
     }
 }
 
-#[cfg(feature = "mania")]
 impl From<mania::ManiaPerformanceAttributes> for PerformanceAttributes {
     #[inline]
     fn from(attributes: mania::ManiaPerformanceAttributes) -> Self {
@@ -618,7 +468,6 @@ impl From<mania::ManiaPerformanceAttributes> for PerformanceAttributes {
     }
 }
 
-#[cfg(feature = "osu")]
 impl From<osu::OsuPerformanceAttributes> for PerformanceAttributes {
     #[inline]
     fn from(attributes: osu::OsuPerformanceAttributes) -> Self {
@@ -626,7 +475,6 @@ impl From<osu::OsuPerformanceAttributes> for PerformanceAttributes {
     }
 }
 
-#[cfg(feature = "taiko")]
 impl From<taiko::TaikoPerformanceAttributes> for PerformanceAttributes {
     #[inline]
     fn from(attributes: taiko::TaikoPerformanceAttributes) -> Self {
@@ -634,7 +482,6 @@ impl From<taiko::TaikoPerformanceAttributes> for PerformanceAttributes {
     }
 }
 
-#[cfg(any(feature = "osu", feature = "taiko"))]
 #[inline]
 fn difficulty_range(val: f64, max: f64, avg: f64, min: f64) -> f64 {
     if val > 5.0 {
@@ -646,13 +493,5 @@ fn difficulty_range(val: f64, max: f64, avg: f64, min: f64) -> f64 {
     }
 }
 
-#[cfg(not(any(
-    feature = "osu",
-    feature = "taiko",
-    feature = "fruits",
-    feature = "mania"
-)))]
-compile_error!("At least one of the features `osu`, `taiko`, `fruits`, `mania` must be enabled");
-
 #[cfg(all(feature = "async_tokio", feature = "async_std"))]
 compile_error!("Only one of the features `async_tokio` and `async_std` should be enabled");
