@@ -1,5 +1,3 @@
-#![cfg(feature = "mania")]
-
 mod gradual_difficulty;
 mod gradual_performance;
 mod pp;
@@ -15,35 +13,109 @@ use crate::{parse::HitObject, Beatmap, GameMode, Mods, Strains};
 const SECTION_LEN: f64 = 400.0;
 const STAR_SCALING_FACTOR: f64 = 0.018;
 
-/// Difficulty calculation for osu!mania maps.
+/// Difficulty calculator on osu!mania maps.
 ///
-/// In case of a partial play, e.g. a fail, one can specify the amount of passed objects.
-pub fn stars(
-    map: &Beatmap,
-    mods: impl Mods,
+/// # Example
+///
+/// ```
+/// use rosu_pp::{ManiaStars, Beatmap};
+///
+/// # /*
+/// let map: Beatmap = ...
+/// # */
+/// # let map = Beatmap::default();
+///
+/// let difficulty_attrs = ManiaStars::new(&map)
+///     .mods(8 + 64) // HDDT
+///     .calculate();
+///
+/// println!("Stars: {}", difficulty_attrs.stars);
+/// ```
+#[derive(Clone, Debug)]
+pub struct ManiaStars<'map> {
+    map: &'map Beatmap,
+    mods: u32,
     passed_objects: Option<usize>,
-) -> ManiaDifficultyAttributes {
-    let mut strain = calculate_strain(map, mods, passed_objects);
+    clock_rate: Option<f64>,
+}
 
-    ManiaDifficultyAttributes {
-        stars: Strain::difficulty_value(&mut strain.strain_peaks) * STAR_SCALING_FACTOR,
+impl<'map> ManiaStars<'map> {
+    /// Create a new difficulty calculator for osu!mania maps.
+    #[inline]
+    pub fn new(map: &'map Beatmap) -> Self {
+        Self {
+            map,
+            mods: 0,
+            passed_objects: None,
+            clock_rate: None,
+        }
+    }
+
+    /// Specify mods through their bit values.
+    ///
+    /// See [https://github.com/ppy/osu-api/wiki#mods](https://github.com/ppy/osu-api/wiki#mods)
+    #[inline]
+    pub fn mods(mut self, mods: u32) -> Self {
+        self.mods = mods;
+
+        self
+    }
+
+    /// Amount of passed objects for partial plays, e.g. a fail.
+    ///
+    /// If you want to calculate the difficulty after every few objects, instead of
+    /// using [`ManiaStars`] multiple times with different `passed_objects`, you should use
+    /// [`ManiaGradualDifficultyAttributes`](crate::mania::ManiaGradualDifficultyAttributes).
+    #[inline]
+    pub fn passed_objects(mut self, passed_objects: usize) -> Self {
+        self.passed_objects = Some(passed_objects);
+
+        self
+    }
+
+    /// Adjust the clock rate used in the calculation.
+    /// If none is specified, it will take the clock rate based on the mods
+    /// i.e. 1.5 for DT, 0.75 for HT and 1.0 otherwise.
+    #[inline]
+    pub fn clock_rate(mut self, clock_rate: f64) -> Self {
+        self.clock_rate = Some(clock_rate);
+
+        self
+    }
+
+    /// Calculate all difficulty related values, including stars.
+    #[inline]
+    pub fn calculate(self) -> ManiaDifficultyAttributes {
+        let mut strain = calculate_strain(self);
+
+        ManiaDifficultyAttributes {
+            stars: Strain::difficulty_value(&mut strain.strain_peaks) * STAR_SCALING_FACTOR,
+        }
+    }
+
+    /// Calculate the skill strains.
+    ///
+    /// Suitable to plot the difficulty of a map over time.
+    #[inline]
+    pub fn strains(self) -> Strains {
+        let clock_rate = self.clock_rate.unwrap_or_else(|| self.mods.clock_rate());
+        let strain = calculate_strain(self);
+
+        Strains {
+            section_length: SECTION_LEN * clock_rate,
+            strains: strain.strain_peaks,
+        }
     }
 }
 
-/// Essentially the same as the [`stars`] function but instead of
-/// evaluating the final strains, it just returns them as is.
-///
-/// Suitable to plot the difficulty of a map over time.
-pub fn strains(map: &Beatmap, mods: impl Mods) -> Strains {
-    let strain = calculate_strain(map, mods, None);
+fn calculate_strain(params: ManiaStars<'_>) -> Strain {
+    let ManiaStars {
+        map,
+        mods,
+        passed_objects,
+        clock_rate,
+    } = params;
 
-    Strains {
-        section_length: SECTION_LEN * mods.speed(),
-        strains: strain.strain_peaks,
-    }
-}
-
-fn calculate_strain(map: &Beatmap, mods: impl Mods, passed_objects: Option<usize>) -> Strain {
     let take = passed_objects.unwrap_or_else(|| map.hit_objects.len());
     let rounded_cs = map.cs.round();
 
@@ -68,7 +140,7 @@ fn calculate_strain(map: &Beatmap, mods: impl Mods, passed_objects: Option<usize
         other => panic!("can not calculate mania difficulty on a {:?} map", other),
     };
 
-    let clock_rate = mods.speed();
+    let clock_rate = clock_rate.unwrap_or_else(|| mods.clock_rate());
     let mut strain = Strain::new(columns);
     let columns = columns as f32;
 
