@@ -1,5 +1,5 @@
 use super::{OsuDifficultyAttributes, OsuPerformanceAttributes, OsuScoreState};
-use crate::{Beatmap, DifficultyAttributes, Mods, PerformanceAttributes};
+use crate::{Beatmap, DifficultyAttributes, Mods, OsuStars, PerformanceAttributes};
 
 /// Performance calculator on osu!standard maps.
 ///
@@ -44,6 +44,7 @@ pub struct OsuPP<'map> {
     pub(crate) n50: Option<usize>,
     pub(crate) n_misses: usize,
     pub(crate) passed_objects: Option<usize>,
+    clock_rate: Option<f64>,
 }
 
 impl<'map> OsuPP<'map> {
@@ -62,6 +63,7 @@ impl<'map> OsuPP<'map> {
             n50: None,
             n_misses: 0,
             passed_objects: None,
+            clock_rate: None,
         }
     }
 
@@ -139,6 +141,16 @@ impl<'map> OsuPP<'map> {
         self
     }
 
+    /// Adjust the clock rate used in the calculation.
+    /// If none is specified, it will take the clock rate based on the mods
+    /// i.e. 1.5 for DT, 0.75 for HT and 1.0 otherwise.
+    #[inline]
+    pub fn clock_rate(mut self, clock_rate: f64) -> Self {
+        self.clock_rate = Some(clock_rate);
+
+        self
+    }
+
     /// Provide parameters through an [`OsuScoreState`].
     #[inline]
     pub fn state(mut self, state: OsuScoreState) -> Self {
@@ -312,10 +324,19 @@ impl<'map> OsuPP<'map> {
 
     /// Calculate all performance related values, including pp and stars.
     pub fn calculate(mut self) -> OsuPerformanceAttributes {
-        let attributes = self
-            .attributes
-            .take()
-            .unwrap_or_else(|| super::stars(self.map, self.mods, self.passed_objects));
+        let attributes = self.attributes.take().unwrap_or_else(|| {
+            let mut calculator = OsuStars::new(self.map).mods(self.mods);
+
+            if let Some(passed_objects) = self.passed_objects {
+                calculator = calculator.passed_objects(passed_objects);
+            }
+
+            if let Some(clock_rate) = self.clock_rate {
+                calculator = calculator.clock_rate(clock_rate);
+            }
+
+            calculator.calculate()
+        });
 
         self.assert_hitresults(attributes).calculate()
     }
@@ -337,41 +358,48 @@ struct OsuPPInner {
 
 impl OsuPPInner {
     fn calculate(mut self) -> OsuPerformanceAttributes {
-        let mut multiplier = 1.12;
+        let (aim_value, speed_value, acc_value, flashlight_value, pp) =
+            if self.total_hits.abs() <= f64::EPSILON {
+                (0.0, 0.0, 0.0, 0.0, 0.0)
+            } else {
+                let mut multiplier = 1.12;
 
-        // NF penalty
-        if self.mods.nf() {
-            multiplier *= (1.0 - 0.02 * (self.effective_misses as f64)).max(0.9);
-        }
+                // NF penalty
+                if self.mods.nf() {
+                    multiplier *= (1.0 - 0.02 * (self.effective_misses as f64)).max(0.9);
+                }
 
-        // SO penalty
-        if self.mods.so() {
-            let n_spinners = self.attributes.n_spinners;
-            multiplier *= 1.0 - (n_spinners as f64 / self.total_hits).powf(0.85);
-        }
+                // SO penalty
+                if self.mods.so() {
+                    let n_spinners = self.attributes.n_spinners;
+                    multiplier *= 1.0 - (n_spinners as f64 / self.total_hits).powf(0.85);
+                }
 
-        // Relax penalty
-        if self.mods.rx() {
-            // * As we're adding 100s and 50s to an approximated number of combo breaks\
-            // * the result can be higher than total hits in specific scenarios
-            // * (which breaks some calculations) so we need to clamp it.
-            self.effective_misses =
-                (self.effective_misses + self.n100 + self.n50).min(self.total_hits as usize);
+                // Relax penalty
+                if self.mods.rx() {
+                    // * As we're adding 100s and 50s to an approximated number of combo breaks\
+                    // * the result can be higher than total hits in specific scenarios
+                    // * (which breaks some calculations) so we need to clamp it.
+                    self.effective_misses = (self.effective_misses + self.n100 + self.n50)
+                        .min(self.total_hits as usize);
 
-            multiplier *= 0.6;
-        }
+                    multiplier *= 0.6;
+                }
 
-        let aim_value = self.compute_aim_value();
-        let speed_value = self.compute_speed_value();
-        let acc_value = self.compute_accuracy_value();
-        let flashlight_value = self.compute_flashlight_value();
+                let aim_value = self.compute_aim_value();
+                let speed_value = self.compute_speed_value();
+                let acc_value = self.compute_accuracy_value();
+                let flashlight_value = self.compute_flashlight_value();
 
-        let pp = (aim_value.powf(1.1)
-            + speed_value.powf(1.1)
-            + acc_value.powf(1.1)
-            + flashlight_value.powf(1.1))
-        .powf(1.0 / 1.1)
-            * multiplier;
+                let pp = (aim_value.powf(1.1)
+                    + speed_value.powf(1.1)
+                    + acc_value.powf(1.1)
+                    + flashlight_value.powf(1.1))
+                .powf(1.0 / 1.1)
+                    * multiplier;
+
+                (aim_value, speed_value, acc_value, flashlight_value, pp)
+            };
 
         OsuPerformanceAttributes {
             difficulty: self.attributes,
